from aiohttp import ClientSession, ClientResponse


class Auth:
    """Class to make authenticated requests."""

    def __init__(self, endpoint: str, token: str):
        """Initialize the auth."""
        self.basepath = endpoint
        self.token = token

    async def request(self, session: ClientSession, method: str, path: str, **kwargs) -> ClientResponse:
        uri = f"{self.basepath}/{path}/"
        return await self.__request(session, method, uri, **kwargs)

    async def request_raw(self, session: ClientSession, method: str, uri: str, **kwargs) -> ClientResponse:
        return await self.__request(session, method, uri, **kwargs)

    async def __request(self, session: ClientSession, method: str, uri: str, **kwargs) -> ClientResponse:
        """Make a request."""
        headers = kwargs.get("headers")
        params = kwargs.get("params")

        if headers is None:
            headers = {}
        else:
            headers = dict(headers)

        if params is None:
            params = {}
        else:
            params = dict(params)

        params["format"] = "json"
        headers["authorization"] = f"Token {self.token}"

        return await session.request(
            method, uri, **kwargs, headers=headers,
        )



from datetime import datetime
from typing import List

from .auth import Auth


class Correspondent:
    """Class that represents a correspondent object in the paperless API."""

    def __init__(self, raw_data: dict, auth: Auth):
        """Initialize a correspondent object."""
        self.raw_data = raw_data
        self.auth = auth

    @property
    def id(self) -> int:
        """Return the id."""
        return self.raw_data["id"]

    @property
    def slug(self) -> str:
        """Return the slug."""
        return self.raw_data["slug"]

    @property
    def name(self) -> str:
        """Return the name."""
        return self.raw_data["name"]

    @property
    def match(self) -> str:
        """Return the match."""
        return self.raw_data["match"]

    @property
    def matching_algorithm(self) -> int:
        """Return the matching_algorithm."""
        return self.raw_data["matching_algorithm"]

    @property
    def is_insensitive(self) -> bool:
        """Return the is_insensitive flag."""
        return self.raw_data["is_insensitive"]

    @property
    def document_count(self) -> int:
        """Return the document_count."""
        return self.raw_data["document_count"]

    @property
    def last_correspondence(self) -> datetime:
        """Return the last_correspondence date."""
        return self.raw_data["last_correspondence"]


class DocumentType:
    """Class that represents a document type object in the paperless API."""

    def __init__(self, raw_data: dict, auth: Auth):
        """Initialize a document type object."""
        self.raw_data = raw_data
        self.auth = auth

    @property
    def id(self) -> int:
        """Return the id."""
        return self.raw_data["id"]

    @property
    def slug(self) -> str:
        """Return the slug."""
        return self.raw_data["slug"]

    @property
    def name(self) -> str:
        """Return the name."""
        return self.raw_data["name"]

    @property
    def match(self) -> str:
        """Return the match."""
        return self.raw_data["match"]

    @property
    def matching_algorithm(self) -> int:
        """Return the matching_algorithm."""
        return self.raw_data["matching_algorithm"]

    @property
    def is_insensitive(self) -> bool:
        """Return the is_insensitive flag."""
        return self.raw_data["is_insensitive"]

    @property
    def document_count(self) -> int:
        """Return the document_count."""
        return self.raw_data["document_count"]


class Tag:
    """Class that represents a tag object in the paperless API."""

    def __init__(self, raw_data: dict, auth: Auth):
        """Initialize a tag object."""
        self.raw_data = raw_data
        self.auth = auth

    @property
    def id(self) -> int:
        """Return the id."""
        return self.raw_data["id"]

    @property
    def slug(self) -> str:
        """Return the slug."""
        return self.raw_data["slug"]

    @property
    def name(self) -> str:
        """Return the name."""
        return self.raw_data["name"]

    @property
    def colour(self) -> int:
        """Return the colour."""
        return self.raw_data["colour"]

    @property
    def match(self) -> str:
        """Return the match."""
        return self.raw_data["match"]

    @property
    def matching_algorithm(self) -> int:
        """Return the matching_algorithm."""
        return self.raw_data["matching_algorithm"]

    @property
    def is_insensitive(self) -> bool:
        """Return the is_insensitive flag."""
        return self.raw_data["is_insensitive"]

    @property
    def is_inbox_tag(self) -> bool:
        """Return the is_inbox_tag flag."""
        return self.raw_data["is_inbox_tag"]

    @property
    def document_count(self) -> int:
        """Return the document_count."""
        return self.raw_data["document_count"]


class SavedView:
    """Class that represents a saved view object in the paperless API."""

    def __init__(self, raw_data: dict, auth: Auth):
        """Initialize a saved view object."""
        self.raw_data = raw_data
        self.auth = auth

    @property
    def id(self) -> int:
        """Return the id."""
        return self.raw_data["id"]

    @property
    def name(self) -> str:
        """Return the name."""
        return self.raw_data["name"]

    @property
    def show_on_dashboard(self) -> bool:
        """Return the show_on_dashboard flag."""
        return self.raw_data["show_on_dashboard"]

    @property
    def show_in_sidebar(self) -> bool:
        """Return the show_in_sidebar flag."""
        return self.raw_data["show_in_sidebar"]

    @property
    def sort_field(self) -> str:
        """Return the sort_field."""
        return self.raw_data["sort_field"]

    @property
    def sort_reverse(self) -> bool:
        """Return the sort_reverse flag."""
        return self.raw_data["sort_reverse"]

    @property
    def filter_rules(self) -> List[dict]:
        """Return the filter_rules dict."""
        return self.raw_data["filter_rules"]


class Document:
    """Class that represents a document object in the paperless API."""

    def __init__(self, raw_data: dict, auth: Auth):
        """Initialize a document object."""
        self.raw_data = raw_data
        self.auth = auth

    @property
    def id(self) -> int:
        """Return the id."""
        return self.raw_data["id"]

    @property
    def correspondent(self) -> int:
        """Return the correspondent id."""
        return self.raw_data["correspondent"]

    @property
    def document_type(self) -> int:
        """Return the document_type id."""
        return self.raw_data["document_type"]

    @property
    def title(self) -> str:
        """Return the title."""
        return self.raw_data["title"]

    @property
    def content(self) -> str:
        """Return the content."""
        return self.raw_data["content"]

    @property
    def tags(self) -> List[int]:
        """Return the list of mapped tags."""
        return self.raw_data["tags"]

    @property
    def created(self) -> datetime:
        """Return the created date."""
        return self.raw_data["created"]

    @property
    def modified(self) -> datetime:
        """Return the modified date."""
        return self.raw_data["modified"]

    @property
    def added(self) -> datetime:
        """Return the added date."""
        return self.raw_data["added"]

    @property
    def archive_serial_number(self) -> str:
        """Return the archive_serial_number."""
        return self.raw_data["archive_serial_number"]

    @property
    def original_file_name(self) -> str:
        """Return the original_file_name."""
        return self.raw_data["original_file_name"]

    @property
    def archived_file_name(self) -> str:
        """Return the archived_file_name."""
        return self.raw_data["archived_file_name"]



from typing import List

from .auth import Auth
from .model import *

import aiohttp


class PaperlessAPI:
    """Class to communicate with the paginated API."""

    def __init__(self, endpoint: str, token: str):
        """Initialize the API and store the auth so we can make token-based requests."""
        self.auth = Auth(endpoint, token)

    async def __get_paginated_results(self, path: str, isa: object) -> List[object]:
        """Walk through the pagination and return a list of objects of type <isa>."""
        json = {}
        results = []

        async with aiohttp.ClientSession() as session:
            while True:
                if not "next" in json:
                    resp = await self.auth.request(session, "get", path)
                else:
                    resp = await self.auth.request_raw(session, "get", json["next"])
                resp.raise_for_status()

                json = dict(await resp.json())

                # extend list
                results.extend([isa(data, self.auth)
                                for data in json["results"]])

                # end loop when no next is given
                if json["next"] is None:
                    break

        return results

    async def get_correspondents(self) -> List[Correspondent]:
        """Return all correspondents."""
        return await self.__get_paginated_results("correspondents", Correspondent)

    async def get_document_types(self) -> List[DocumentType]:
        """Return all document types."""
        return await self.__get_paginated_results("document_types", DocumentType)

    async def get_tags(self) -> List[Tag]:
        """Return all tags."""
        return await self.__get_paginated_results("tags", Tag)

    async def get_saved_views(self) -> List[SavedView]:
        """Return all saved views."""
        return await self.__get_paginated_results("saved_views", SavedView)

    async def get_documents(self) -> List[Document]:
        """Return all documents."""
        return await self.__get_paginated_results("documents", Document)
